@@ -25,14 +25,13 @@ import re
 import warnings
 
 import abc
+from debtcollector import removals
 import netaddr
 import rfc3986
-import six
-from six.moves import range as compat_range
 
 
-@six.add_metaclass(abc.ABCMeta)
-class ConfigType(object):
+class ConfigType(object, metaclass=abc.ABCMeta):
+
     def __init__(self, type_name='unknown type'):
         self.type_name = type_name
 
@@ -43,7 +42,10 @@ class ConfigType(object):
 
         """
         if sample_default is not None:
-            default_str = sample_default
+            if isinstance(sample_default, str):
+                default_str = sample_default
+            else:
+                default_str = self._formatter(sample_default)
         elif default is None:
             default_str = self.NONE_DEFAULT
         else:
@@ -51,9 +53,9 @@ class ConfigType(object):
         return [default_str]
 
     def quote_trailing_and_leading_space(self, str_val):
-        if not isinstance(str_val, six.string_types):
+        if not isinstance(str_val, str):
             warnings.warn('converting \'%s\' to a string' % str_val)
-            str_val = six.text_type(str_val)
+            str_val = str(str_val)
         if str_val.strip() != str_val:
             return '"%s"' % str_val
         return str_val
@@ -131,7 +133,7 @@ class String(ConfigType):
             re_flags = re.IGNORECASE if self.ignore_case else 0
 
             # Check if regex is a string or an already compiled regex
-            if isinstance(regex, six.string_types):
+            if isinstance(regex, str):
                 self.regex = re.compile(regex, re_flags)
             else:
                 self.regex = re.compile(regex.pattern, re_flags | regex.flags)
@@ -140,7 +142,7 @@ class String(ConfigType):
         value = str(value)
         if self.quotes and value:
             if value[0] in "\"'":
-                if value[-1] != value[0]:
+                if len(value) == 1 or value[-1] != value[0]:
                     raise ValueError('Non-closed quote: %s' % value)
                 value = value[1:-1]
 
@@ -457,7 +459,8 @@ class List(ConfigType):
     comma and next item until validation succeeds or there is no parts left.
     In the later case it will signal validation error.
 
-    :param item_type: type of list items
+    :param item_type: Type of list items. Should be an instance of
+                      ``ConfigType``.
     :param bounds: if True, value should be inside "[" and "]" pair
     :param type_name: Type name to be used in the sample config file.
 
@@ -479,9 +482,9 @@ class List(ConfigType):
 
     def __call__(self, value):
         if isinstance(value, (list, tuple)):
-            return list(six.moves.map(self.item_type, value))
+            return list(map(self.item_type, value))
 
-        s = value.strip()
+        s = value.strip().rstrip(',')
         if self.bounds:
             if not s.startswith('['):
                 raise ValueError('Value should start with "["')
@@ -525,14 +528,16 @@ class List(ConfigType):
         )
 
     def _formatter(self, value):
-        if isinstance(value, six.string_types):
-            return value
+        fmtstr = '[{}]' if self.bounds else '{}'
+        if isinstance(value, str):
+            return fmtstr.format(value)
         if isinstance(value, list):
             value = [
-                six.text_type(v)
+                self.item_type._formatter(v)
                 for v in value
             ]
-        return ','.join(value)
+            return fmtstr.format(','.join(value))
+        return fmtstr.format(self.item_type._formatter(value))
 
 
 class Range(ConfigType):
@@ -578,7 +583,7 @@ class Range(ConfigType):
             step = -1
         if self.inclusive:
             right += step
-        return compat_range(left, right, step)
+        return range(left, right, step)
 
     def __eq__(self, other):
         return (
@@ -803,7 +808,7 @@ class Hostname(ConfigType):
         return value
 
 
-class HostAddress(object):
+class HostAddress(ConfigType):
     """Host Address type.
 
     Represents both valid IP addresses and valid host domain names
@@ -821,6 +826,7 @@ class HostAddress(object):
 
         """
 
+        super(HostAddress, self).__init__(type_name=type_name)
         self.ip_address = IPAddress(version, type_name)
         self.hostname = Hostname('localhost')
 
@@ -876,31 +882,54 @@ class URI(ConfigType):
         self.schemes = schemes
 
     def __call__(self, value):
-        if not rfc3986.is_valid_uri(value, require_scheme=True,
-                                    require_authority=True):
-            raise ValueError('invalid URI: %r' % value)
+        uri = rfc3986.uri_reference(value)
+        validator = rfc3986.validators.Validator().require_presence_of(
+            'scheme', 'host',
+        ).check_validity_of(
+            'scheme', 'host', 'path',
+        )
+        if self.schemes:
+            validator = validator.allow_schemes(*self.schemes)
+        try:
+            validator.validate(uri)
+        except rfc3986.exceptions.RFC3986Exception as exc:
+            raise ValueError(exc)
 
         if self.max_length is not None and len(value) > self.max_length:
             raise ValueError("Value '%s' exceeds maximum length %d" %
                              (value, self.max_length))
 
-        if self.schemes:
-            scheme = rfc3986.uri_reference(value).scheme
-            if scheme not in self.schemes:
-                raise ValueError("URI scheme '%s' not in %s" %
-                                 (scheme, self.schemes))
-
-        self.value = value
+        # NOTE(dhellmann): self.value is deprecated, and we don't want
+        # to trigger a deprecation warning ourselves so we modify
+        # self._value directly.
+        self._value = value
         return value
 
+    @removals.removed_property
+    def value(self):
+        return self._value
+
+    @value.setter
+    def value(self, newval):
+        self._value = newval
+
+    @value.deleter
+    def value(self):
+        del self._value
+
     def __repr__(self):
         return 'URI'
 
     def __eq__(self, other):
-        return (
-            (self.__class__ == other.__class__) and
-            (self.value == other.value)
+        to_compare = ['__class__', 'max_length', 'schemes']
+        unset = object()
+        my_values = tuple(
+            getattr(self, name, unset) for name in to_compare
         )
+        other_values = tuple(
+            getattr(other, name, unset) for name in to_compare
+        )
+        return my_values == other_values
 
     def _formatter(self, value):
         return value
